#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

/*  Loads a Flac file and does a statistical analysis on the sample
    values it contains.
    
     */




#define SWAP(a,b) tempr=(a) ; (a)=(b) ; (b)=tempr

int ok, infileok;

FILE* infile;
FILE* tfile;
FILE* outfile;
FILE* specfile;
FILE* vin;
FILE* vout;
FILE* settingsfile;

unsigned char settingsfilename[1024];
unsigned char outdirname[1024];
unsigned char resdirname[1024];
unsigned char infilename[1024];
unsigned char outfilename[1024];
unsigned char specinputname[1024];
unsigned char appdirname[1024];
unsigned char tempfilename[1024];
unsigned char leafname[512];
unsigned char specfilename[1024];
unsigned char commandstring[2048];
unsigned char vlabeltext[256];

unsigned char chbuffer[10];

unsigned char* cpointer;

int ch_channels, ch_bitspersample, ch_samplerate, ch_bytes_remaining;
int number_full_chunks, chunks_done;
int bits_per_sample,sample_rate,blocksize,chunksize;
int ifind;

double ch_duration_secs;

double xleft[4096],xright[4096], weight[4096];
double x[8192];  /* double size for imag zeros */
double pout_left[2048], pout_right[2048];

short dbuffer[5770000];  /* note buffer oversized in case of tail error */
int left_hist[65537], right_hist[65537];

int left_hist_mid[4097], right_hist_mid[4097];


double pi,pi2;
int i,il,ir,id,ispec0;
int n,nn,npoints,nhalf;

int fmins,fsecs,fiok,istop,timelimit;
int itop,izero,nsamples_per_channel,dco_left,dco_right;
int zero24_count_left,zero24_count_right;

double bscale,hscale,hscale24;

void read_settingsfile(void);
int read_header(void);
int getbytes(int);
void process_file(void);
void remove_tempfile(void);
int set_rate_and_bits(int);
int get_flac_chunk(int);
void process_chunk16(void);
void process_chunk24(void);
void process_results(void);
void report_histograms(void);
void do_spectra(void);
void generate_apodise_weights(void);
void loadnorm(void);
void apodise(void);
void load_fft_array(double*);
void fft(void);
void bit_rev(void);
void save_ampspectrum(double*);
void save_outputspectra(void);
void output_veusz(void);
void veusz_label(char*,char*,double,double);



int main(int argc,char* argv[])
{
  
  
  printf("  Flac Health Check Ver 0.64 23rd May 2015\n");
  printf("  ========================================\n\n");
  printf("This lets you do a statistical analysis on a stereo Flac file\n");
  printf("Please read the Help inside the application before first use.\n");
  
  sprintf(appdirname,"%s",argv[0]);
  ok=(long)strrchr(appdirname,'/')-(long)appdirname;
  appdirname[ok]=(char)0;
  printf("Application directory = %s\n",appdirname);
  
  read_settingsfile();
  
  if (argc>1)
  {
    infileok=1;
    sprintf(infilename,"%s",argv[1]);
    printf("Input from file = %s\n\n",infilename);
  }
  else
  {
    infileok=0;
    printf("\nDrag a Flac file and drop it on the application icon\n");
    printf("if you wish to examine the contents of the file.\n\n");
  }
  
  if(infileok==1)
  {
    ok=(long)strrchr(infilename,'/');
    sprintf(leafname,"%s",(char*)(ok+1));
    printf("Leafname with extension = %s\n",leafname);
    
    ok=(long)strrchr(leafname,'.');
    
    printf("ext char at %d\n",ok);
    
    if(ok==0)
    {
      printf("No filetype extension found! Is it a flac file?\n");
      infileok=0;
    }
    else
    {
      ok-=(long)leafname;
      leafname[ok]=(char)0;
      printf("Leafname without extension = %s\n",leafname);
      sprintf(outdirname,"%s/Results/%s",appdirname,leafname);
      printf("Output directory = %s\n",outdirname);
      
      
      
      printf("Input file %s\n",infilename);
      
      if(read_header()==1)
      {
         
         printf("Temp file blocksize = %d\n",blocksize);
         
         sprintf(commandstring,"mkdir -p %s",outdirname);
         printf("command is %s\n",commandstring);
         system(commandstring);
         
         if(set_rate_and_bits(blocksize)>0)
         {
         
              process_file();
              
              process_results();
              report_histograms();
              do_spectra();
              output_veusz();
         }
 
      }
      
    }
  
  }
  
  printf("\nDone!\n");
  
  return 0;
}

void read_settingsfile(void)
{
  
  sprintf(settingsfilename,"%s/Settings",appdirname);
  printf("\nSettings file at %s\n",settingsfilename);

  settingsfile=fopen(settingsfilename,"rt");
  fscanf(settingsfile,"%s",tempfilename);
  printf("read %s\n",tempfilename);
  fclose(settingsfile);
  sprintf(resdirname,"%s/Results",appdirname);
  printf("Results directory %s\n",resdirname);
  
  if(tempfilename[0]=='*')
  {
    sprintf(tempfilename,"%s/tempfile",resdirname);
  }
  
  
  
  
  printf("Tempfile will be %s\n",tempfilename);
  
  
  
  
  
}


void veusz_label(char* label_text,char* label_name,double xlpos,double ylpos)
{
   fprintf(vout,"To('..')\n");
   fprintf(vout,"Add('label', name='%s',autoadd=False)\n",label_name);
   fprintf(vout,"To('%s')\n",label_name);
   fprintf(vout,"Set('label', u'%s')\n",label_text);
   fprintf(vout,"Set('xPos', [%7.4lf])\n",xlpos);
   fprintf(vout,"Set('yPos', [%7.4lf])\n",ylpos);

}

void output_veusz(void)
{
  char vinname[1024];
  char voutname[1024];
  char vchar;
  
  int vcheck, vcount;
  
  
  sprintf(vinname,"%s/spectrum_style.vsz",appdirname);
  /* printf("Style from %s\n",vinname); */
  sprintf(voutname,"%s/load_intervalspectrum.vsz",outdirname);
  /* printf("Interval loader file %s\n",voutname); */
  vin=fopen(vinname,"rt");  vout=fopen(voutname,"wt");
  vcount=0;
  do
  {
    vcheck=(long)fscanf(vin,"%c",&vchar); 
    if (vcheck==1) fprintf(vout,"%c",vchar);
    vcount++;
    if(vcount>10000) vcheck=0;
  } while (vcheck==1);
  /* printf(" got %d",vcount); */
  fclose(vin);
  sprintf(vlabeltext,"Results for:   %s.flac",leafname);
  veusz_label(vlabeltext,"label1",0.14,0.91);
  sprintf(vlabeltext,"sample rate = %d   bits/sample = %d",ch_samplerate,bits_per_sample);
  veusz_label(vlabeltext,"label2",0.14,0.86);
  fclose(vout);
  
  sprintf(vinname,"%s/normstats_style.vsz",appdirname);
  sprintf(voutname,"%s/load_normstats.vsz",outdirname);
  /* printf("From %s\n to %s\n",vinname,voutname); */
  vin=fopen(vinname,"rt");  vout=fopen(voutname,"wt");
  vcount=0;
  do
  {
    vcheck=(long)fscanf(vin,"%c",&vchar); 
    if (vcheck==1) fprintf(vout,"%c",vchar);
    vcount++;
    if(vcount>10000) vcheck=0;
  } while (vcheck==1);
  
  fclose(vin);
  sprintf(vlabeltext,"Results for:   %s.flac",leafname);
  veusz_label(vlabeltext,"label1",0.14,0.91);
  sprintf(vlabeltext,"sample rate = %d   bits/sample = %d",ch_samplerate,bits_per_sample);
  veusz_label(vlabeltext,"label2",0.14,0.86);
  fclose(vout);
  
  
  sprintf(vinname,"%s/32blocked_style.vsz",appdirname);
  sprintf(voutname,"%s/load_32blocked_samplestats.vsz",outdirname);
  /* printf("From %s\n to %s\n",vinname,voutname); */
  vin=fopen(vinname,"rt");  vout=fopen(voutname,"wt");
  vcount=0;
  do
  {
    vcheck=(long)fscanf(vin,"%c",&vchar); 
    if (vcheck==1) fprintf(vout,"%c",vchar);
    vcount++;
    if(vcount>10000) vcheck=0;
  } while (vcheck==1);
  
  fclose(vin);
  sprintf(vlabeltext,"Results for:   %s.flac",leafname);
  veusz_label(vlabeltext,"label1",0.14,0.91);
  sprintf(vlabeltext,"sample rate = %d   bits/sample = %d",ch_samplerate,bits_per_sample);
  veusz_label(vlabeltext,"label2",0.14,0.86);
  fclose(vout);
  
  
}




void do_spectra(void)
{
  
  generate_apodise_weights();
  loadnorm();
  apodise();
  load_fft_array(xleft);
  fft();
  save_ampspectrum(pout_left);
  load_fft_array(xright);
  fft();
  save_ampspectrum(pout_right);
  save_outputspectra();
}


void save_outputspectra(void)
{
  int q;
  double f,f0;
  
  sprintf(specfilename,"%s/intervalspectrum.csv",outdirname);
  f0=1.0/4096.0;
  specfile=fopen(specfilename,"wt");
  
  fprintf(specfile," \"interval\", \"left\", \"right\",\n");
  
  q=nhalf-1;
  do
  {
    f=f0*(double)q;
    fprintf(specfile,"%8.4lf,  %7.3lf,  %7.3lf,\n",1.0/f,pout_left[q],pout_right[q]);
    q--;
  } while (q>1);
  
  fclose(specfile);
}

void apodise(void)
{
  int iap;
  iap=0;
  do
  {
    xleft[iap]=weight[iap]*xleft[iap];
    xright[iap]=weight[iap]*xright[iap];
    iap++;
  } while (iap<4096);
}


void save_ampspectrum(double* onearray)
{
  int icount,idc;
  double p1,p2,specscale;
  specscale=1.0/4096.0;
  
  icount=0; idc=1;
  do
  {
    p1=x[idc];
    idc++;
    p2=x[idc];
    idc++;
    p1=p1*p1+p2*p2;
    
    if(p1>0.0) p1=sqrt(p1);
    onearray[icount]=specscale*p1;
    icount++;
    
  } while (icount<nhalf);
  
}

void generate_apodise_weights(void)
{
  int iwf,iwb;
  double weight_now,wscale,xplace;
  FILE* repfile;
  iwf=0; iwb=4095; wscale=0.0;
  
    do
    {
      xplace=(double)iwf;
      xplace=pi2*xplace/4095.0;
      weight_now= 0.35875-0.48829*cos(xplace)+0.14128*cos(2.0*xplace)-0.01168*cos(3.0*xplace);
      weight[iwf]=weight_now;
      weight[iwb]=weight_now;
      wscale+=weight_now;
      iwf++;
      iwb--;
    } while (iwf<iwb);
    
    wscale=wscale/((double)(iwf-1));
    
    iwf=0;
    do
    {
      weight[iwf]=weight[iwf]/wscale;
      iwf++;
    } while (iwf<4096);
  
}

void load_fft_array(double* inarray)
{
  int icount,icd;
  icount=0; icd=1;
  x[0]=0.0;
  do
  {
    x[icd]=inarray[icount];
    icd++;
    x[icd]=0.0;
    icd++;
    icount++;
  } while (icount<npoints);
}

void loadnorm(void)
{
  double a,b;
  unsigned char lcha;
  
  printf("Doing interval spectrum of %s\n",specinputname);
  infile=fopen(specinputname,"rt");
  
  do
  {
    fscanf(infile,"%c",&lcha);
  } while (15 < (int)lcha);
  
  
  i=0;
  do
  {
    fscanf(infile,"%d,%lf,%lf,",&id,&a,&b);
    xleft[i]=a; xright[i]=b;
    /*
    if(i==0)
    {
      printf("first line %6.8lf %6.8lf\n",a,b);
    }
    */
    if(id==0)
    {
      ispec0=i;
      /* printf("Spectrum zero spike at %d  %8.3lf %8.3lf\n",ispec0,xleft[i],xright[i]); */
    }
    
    i++;
  } while(i<4096);
  
  fclose(infile);
  
  xleft[ispec0]  = 0.25*( xleft[ispec0-2] +  xleft[ispec0-1] +  xleft[ispec0+1] +  xleft[ispec0+2]);
  xright[ispec0] = 0.25*(xright[ispec0-2] + xright[ispec0-1] + xright[ispec0+1] + xright[ispec0+2]);
  
  /* printf("Spectrum zero spike reduced to  %8.3lf %8.3lf\n",xleft[ispec0],xright[ispec0]); */
  
  
  
}

void fft(void)
{
  int mmax,istep,m,i,j;
  int ibig,ismall,jbig,jsmall;
  double theta,wpi,wpr;
  double wi,wr,tr,ti,wtemp;
  
  ibig=-99;  jbig=-99;
  ismall=99; jsmall=99;

  mmax=2;
  theta=2.0*pi/mmax;
  
  bit_rev();
  
  while ( nn > mmax )
  {
   istep=2*mmax;
   theta=2.0*pi/mmax;
   wpr=-2.0*sin(0.5*theta)*sin(0.5*theta);
   wpi=sin(theta);
   wr=1.0;
   wi=0.0;
   m=1;
   do
   {
     i=m;
     do
     {
       j=i+mmax;
       
       if(i>ibig)
       {
         ibig=i;
       }
       if(i<ismall)
       {
         ismall=i;
       }
       if (j>jbig)
       {
         jbig=j;
       }
       if (j<jsmall)
       {
         jsmall=j;
       }
       tr=wr*x[j]-wi*x[j+1];
       ti=wr*x[j+1]+wi*x[j];
       x[j]=x[i]-tr;
       x[j+1]=x[i+1]-ti;
       x[i]=x[i]+tr;
       x[i+1]=x[i+1]+ti;
       i+=istep;
     } while ( i < nn );
     wtemp=wr;
     wr=wr*wpr-wi*wpi+wr;
     wi=wi*wpr+wtemp*wpi+wi;
     m+=2;
   } while ( m < mmax );
   mmax=istep;   
  }
}


void bit_rev(void)
{
  int ibr,jbr,mbr;
  double tr,ti;
  jbr=1;
  ibr=1;
  do
  {
    if ( jbr > ibr )
    {
      tr=x[jbr];
      ti=x[jbr+1];
      x[jbr]=x[ibr];
      x[jbr+1]=x[ibr+1];
      x[ibr]=tr;
      x[ibr+1]=ti;
    }
    mbr=nn/2;
    while ( mbr >= 2 && jbr > mbr )
    {
      jbr-=mbr;
      mbr=mbr/2;
    }
    jbr=jbr+mbr;   
    ibr+=2;
  } while ( ibr < nn );
}

void report_histograms(void)
{
  /* This procedure reports the histograms collected */
  
  
  int aleft,aright,i32;
  int ih,isavelimit,ib;
  double rangescale;
  printf("Saving results\n");
  
  ih=0; isavelimit=65536; rangescale=1.0/32768.0;
  
  ib=1;
  if (bits_per_sample==24) ib=256;
  

  sprintf(outfilename,"%s/left_samplestats",outdirname);
  outfile=fopen(outfilename,"wt");
  do
  {
    fprintf(outfile,"%6d, %8d,\n",ib*(ih-izero),left_hist[ih]);
    ih++;
  } while (ih<isavelimit);
  fclose(outfile);
  
  /* ih=izero/2; */
  ih=0;
  
  sprintf(outfilename,"%s/right_samplestats",outdirname);
  outfile=fopen(outfilename,"wt");
  do
  {
    fprintf(outfile,"%6d, %8d,\n",ib*(ih-izero),right_hist[ih]);
    ih++;
  } while (ih<isavelimit);
  fclose(outfile);
  
  sprintf(outfilename,"%s/normstats.csv",outdirname);
  sprintf(specinputname,"%s",outfilename);
  
  
  
  outfile=fopen(outfilename,"wt");
  fprintf(outfile," \"sval\", \"left\", \"right\",\n");
  
  
  if(bits_per_sample==16)
  {
    ih=izero-2048; isavelimit=izero+2048;
    
    do
    {
      fprintf(outfile,"%6d, %6.4lf, %6.4lf,\n",ih-izero,hscale*(double)left_hist[ih],hscale*(double)right_hist[ih]  );
      ih++;
    } while (ih<isavelimit);
  }
  
  if(bits_per_sample==24)
  {
    ih=0; hscale24=hscale*256.0;
    
    do
    {
      fprintf(outfile,"%6d, %6.4lf, %6.4lf,\n",ih-2048,hscale24*(double)left_hist_mid[ih],hscale24*(double)right_hist_mid[ih]  );
      ih++;
    } while (ih<4096);
    
    
  }
  
  fclose(outfile);
  
  ih=0; isavelimit=itop-1;

  sprintf(outfilename,"%s/32blocked_samplestats.csv",outdirname);
  outfile=fopen(outfilename,"wt");
  fprintf(outfile," \"sval\", \"left\", \"right\",\n");
  do
  {
    
    i32=0; aleft=0; aright=0;
    do
    {
      
      aright+=right_hist[ih];
      aleft+=left_hist[ih];
      ih++;
      i32++;
    } while (i32<32);
    
    
    fprintf(outfile,"%9.6lf, %7.3lf, %7.3lf,\n",rangescale*(double)(ih-izero-16),hscale*((double)aleft)/32.0,hscale*((double)aright)/32.0);
    
    
  } while (ih<izero);
  
  /* now do center zero value */
  
  
  fprintf(outfile,"%9.6lf, %7.3lf, %7.3lf,\n",rangescale*(double)(ih-izero),hscale*(double)left_hist[ih],hscale*(double)right_hist[ih]);
  ih++;
  
  
  do
  {
    
    i32=0; aleft=0; aright=0;
    do
    {
      
      aright+=right_hist[ih];
      aleft+=left_hist[ih];
      ih++;
      i32++;
    } while (i32<32);
    
    
    fprintf(outfile,"%9.6lf, %7.3lf, %7.3lf,\n",rangescale*(double)(ih-izero-17),hscale*((double)aleft)/32.0,hscale*((double)aright)/32.0);
    
    
  } while (ih<itop);
  
  fclose(outfile);

}



void process_results(void)
{
  int ih,icy;
  double aleft,aright,blackleft,blackright;
  
  printf("Process results\n");
  
  bscale=(double)nsamples_per_channel;
  
  printf(" %d samples per channel in file\n",(int)bscale);
  
  hscale = (double)itop;
  hscale = hscale/bscale;
  printf(" %6.4lf scaling\n",hscale);
  
  ih=izero-5; aleft=0.0; aright=0.0; icy=0;
  do
  {
    aleft  += (double)left_hist[ih];
    aright += (double)right_hist[ih];
    icy++;
    ih++;
  } while (ih<izero);
  ih++;
  
  do
  {
    aleft  += (double)left_hist[ih];
    aright += (double)right_hist[ih];
    
    icy++;
    ih++;
  } while (ih <= (izero+5));
  
  aleft = aleft/(double)icy; aright = aright/(double)icy;
  
  printf("Near zero ave sample count levels = %6.4lf %6.4lf\n",hscale*aleft,hscale*aright);
  
  blackleft=hscale*((double)left_hist[izero]-aleft);
  blackright=hscale*((double)right_hist[izero]-aright);
  
  printf("Normalised digital black counts approx L= %6.1lf  R= %6.1lf\n",blackleft,blackright);
  
  blackleft=((double)left_hist[izero]-aleft)/(double)sample_rate;
  blackright=((double)right_hist[izero]-aright)/(double)sample_rate;
  
  printf("Corresponds to L= %6.2lf  R= %6.2lf seconds of digital black silence.\n",blackleft,blackright);
  
  printf(" DC offsets L= %5.2lf  R= %5.2lf sample value\n",((double)dco_left)/bscale,((double)dco_right)/bscale);
  
  
  
  
}


void process_file(void)
{
  printf("processing content of file\n");
  
  fmins=0; fsecs=0;
  
  itop=65536; izero=itop/2; nsamples_per_channel=0;
  dco_left=0; dco_right=0;
  hscale=(double)itop;
  
  zero24_count_left=0; zero24_count_right=0;
  pi=3.1415927;
  pi2=3.1415927*2.0;
  n=4096; nn=2*n; npoints=n; nhalf=n/2;
  
  chunks_done=0; istop=0;
  
  
  remove_tempfile();
  
  while(0==get_flac_chunk(5))
  {
    tfile=fopen(tempfilename,"rd");
    chunksize=fread(dbuffer,1,blocksize,tfile);
    fclose(tfile);
    remove_tempfile();
    /* printf("size got %d\n",chunksize); */
    
    fsecs+=5;
    printf("-");
    if(fsecs>=60)
    {
      fsecs-=60;
      fmins++;
      printf(" %02dm\n",fmins);
    }
    
    if(bits_per_sample==16)
    {
      process_chunk16();
    }
        
    if(bits_per_sample==24)
    {
      process_chunk24();
    }
     
        
    chunks_done++;
    /* printf("chunks done %d\n",chunks_done); */
  }
  
  printf("\n All the full 5 sec chunks done!\nDoing any tail\n");
  
  if(0==get_flac_chunk(-1))
  {
    
    tfile=fopen(tempfilename,"rd");
    chunksize=fread(dbuffer,1,blocksize,tfile);
    fclose(tfile);
    /* printf("size got %d\n",chunksize); */
    remove_tempfile();
    
    if(bits_per_sample==16)
    {
      process_chunk16();
    }
        
    if(bits_per_sample==24)
    {
      process_chunk24();
    }
      
    
  }
    
   
  printf("Number samples/channel examined = %d\n",nsamples_per_channel);
     
  
  
}



void process_chunk24(void)
{
  int ii,iok,iz,ispc;
  int icut,icut2,lt,rt;
  
  icut=32768*256; icut2=icut*2;
  
  
  ii=0; ispc=chunksize;
  
  /* printf("process 24 on %d bytes equiv to samples = %d samples in block\n",ispc,ispc/3); */
  
  cpointer=(unsigned char*)dbuffer;
  
  do
  {
    iok=1;
    
    lt=65536*(int)cpointer[ii+2] + 256*(int)cpointer[ii+1] + (int)cpointer[ii];
    ii+=3;
    rt=65536*(int)cpointer[ii+2] + 256*(int)cpointer[ii+1] + (int)cpointer[ii];
    ii+=3;
    if (lt>=icut) lt-=icut2;
    if (rt>=icut) rt-=icut2;
    
    dco_left+=lt;  dco_right+=rt;
    
    if(lt==0) zero24_count_left++;
    if(rt==0) zero24_count_right++;
    
    if( (lt<2048)&&(lt>=(-2048)))
    {
      left_hist_mid[lt+2048] ++;
    }
    
    if( (rt<2048)&&(rt>=(-2048)))
    {
      right_hist_mid[rt+2048] ++;
    }
    
    if (lt>0)
    {
      lt += 128;
    }
    else
    {
      lt -= 128;
    }
    
    iz=izero+lt/256;
    
    if(iz<0)
    {
      iok=0;
      printf("left under zero\n");
    }
    if(iz>=itop)
    {
      iok=0;
      printf("left over top\n");
    }
    if(iok==1)
    {
      left_hist[iz] +=1;
    }
      
    
    iz=izero+rt/256;
    
    
    if(iz<0)
    {
      iok=0;
      printf("right under zero\n");
    }
    if(iz>=itop)
    {
      iok=0;
      printf("right over top\n");
    }
    if(iok==1)
    {
      right_hist[iz] +=1;
    } 
    
    
    
    nsamples_per_channel++;
    
    
    
    
  } while(ii<ispc);
  
  /* printf(" ***24 done %d samples per channel so far\n\n",nsamples_per_channel); */
  
}

void process_chunk16(void)
{
  int ii,iok,iz,ispc;
  
  
  ii=0; ispc=chunksize/2;
  
 /* printf("process 16 on %d samples\n",ispc); */
  
   do
   {
      iok=1;
      iz=izero + (int)dbuffer[ii];
      
      dco_left+=(int)dbuffer[ii];
      
      if(iz<0)
      {
        iok=0;
        printf("left under zero\n");
      }
      if(iz>=itop)
      {
        iok=0;
        printf("left over top\n");
      }
      if(iok==1)
      {
        left_hist[iz] +=1;
      }
      
      ii++;
      
     iok=1;
      iz=izero + (int)dbuffer[ii];
      
      dco_right+=(int)dbuffer[ii];
      
      if(iz<0)
      {
        iok=0;
        printf("right under zero\n");
      }
      if(iz>=itop)
      {
        iok=0;
        printf("right over top\n");
      }
      if(iok==1)
      {
        right_hist[iz] +=1;
      } 
      ii++;
      
      nsamples_per_channel++;
      
   } while (ii<ispc);
  
  
  /* printf(" *** done %d samples per channel so far\n\n",nsamples_per_channel); */
  
  
}






int get_flac_chunk(int istep)
{
  int retcode;
  if(istep>0)
  {
    sprintf(commandstring,"flac -d %s --skip %02d:%02d --until +00:05 --force-raw-format --endian=little --sign=signed --totally-silent -f -o %s",infilename,fmins,fsecs,tempfilename);
  }
  
  if(istep<0)
  {
    sprintf(commandstring,"flac -d %s --skip %02d:%02d --force-raw-format --endian=little --sign=signed --totally-silent -f -o %s",infilename,fmins,fsecs,tempfilename);
    /* printf("tail command =\n %s\n\n",commandstring); */
  }
  
  /* printf("get chunk command =\n %s  istep %d  samples %d\n\n",commandstring,istep,nsamples_per_channel); */
  
  retcode=system(commandstring);
  
  
  return retcode;
}


void remove_tempfile(void)
{
  sprintf(commandstring,"rm -f %s",tempfilename);
  
  /* printf("remove tempfile using %s\n",commandstring); */
  system(commandstring);
  
}


int set_rate_and_bits(int fin)
{
  bits_per_sample=0;
  
  switch(fin)
  {
    case 882000:
      bits_per_sample=16;
      sample_rate=44100;
      break;
    case 960000:
      bits_per_sample=16;
      sample_rate=48000;
      break;
    case 1323000:
      bits_per_sample=24;
      sample_rate=44100;
      break;
    case 1440000:
      bits_per_sample=24;
      sample_rate=48000;
      break;
    
    case 1764000:
      bits_per_sample=16;
      sample_rate=88200;
      break;
    case 1920000:
      bits_per_sample=16;
      sample_rate=96000;
      break;
    case 2646000:
      bits_per_sample=24;
      sample_rate=88200;
      break;
    case 2880000:
      bits_per_sample=24;
      sample_rate=96000;
      break;
    
    case 3528000:
      bits_per_sample=16;
      sample_rate=176400;
      break;
    case 3840000:
      bits_per_sample=16;
      sample_rate=192000;
      break;
    case 5292000:
      bits_per_sample=24;
      sample_rate=176400;
      break;
    case 5760000:
      bits_per_sample=24;
      sample_rate=192000;
      break;
  }
      
  printf("File recognised as %d bits per sample  %d samples per sec.\n",bits_per_sample,sample_rate);
  
  return bits_per_sample;
}




int read_header(void)
{
  int chok;
  
  int chai,chbi;
  
  chok=1;
  infile=fopen(infilename,"rd");
  chai=getbytes(4);
  /* printf(" %d bytes read => start %s\n",chai,chbuffer); */
  
  if(strcmp(chbuffer,"fLaC")==0)
  {
    printf("flac file OK\n");
    getbytes(1);
    chai=128&(int)chbuffer[0];
    chbi=127&(int)chbuffer[0];
    /* printf(" %d %d\n",chai,chbi); */
    if (chai==0)
    {
      printf("NOT last block of metadata\n");
    }
    else
    {
      printf("First and last block of metadata\n");
    }
    if(chbi==0)
    {
      printf("This block IS streaminfo\n");
    }
    else
    {
      chok=0;
    }
    if(chok==1)
    {
      printf("\n   *** Examine Flac file header \n");
      getbytes(3);  chai=65536*(int)chbuffer[0] + 256*(int)chbuffer[1] + (int)chbuffer[2];
      printf("Length of metadata block %d\n",chai);
      getbytes(2);  chai=256*(int)chbuffer[0] + (int)chbuffer[1];
      printf("Min block size = %d ",chai);
      getbytes(2);  chai=256*(int)chbuffer[0] + (int)chbuffer[1];
      printf("Max block size = %d \n",chai);
      getbytes(3);  chai=65536*(int)chbuffer[0] + 256*(int)chbuffer[1] + (int)chbuffer[2];
      printf("Min frame size %d ",chai);
      getbytes(3);  chai=65536*(int)chbuffer[0] + 256*(int)chbuffer[1] + (int)chbuffer[2];
      printf("Max frame size %d\n",chai);
      getbytes(3);  chai=65536*(int)chbuffer[0] + 256*(int)chbuffer[1] + (int)chbuffer[2];
      /* printf("20bit mix %d\n",chai); */
      ch_samplerate=(chai/16)&1048575;
      printf("Sample rate = %d  ",ch_samplerate);
      
      ch_channels= 1 + ((chai/2)&0x111);
      printf("Number of channels %d \n",ch_channels);
      
      chbi=((int)chbuffer[2])&1;
      printf("Bits per sample carry %d  ",chbi);
      
      getbytes(5); 
      chai=((int)chbuffer[0])/16  ;
      ch_bitspersample=chbi*16+chai+1;
      printf("Bits per sample %d\n",ch_bitspersample);
      
      chbi=16777216*(int)chbuffer[1] + 65536*(int)chbuffer[2] + 256*(int)chbuffer[3] + (int)chbuffer[3];
      
      printf("Total samples per channel in file %d (approx.)\n",chbi);
      
      ch_bytes_remaining=(chbi*ch_channels*ch_bitspersample)/8;
      ch_duration_secs=((double)chbi)/(double)ch_samplerate;
      
      printf("Duration = %7.1lf sec\n",ch_duration_secs);
      
      
      number_full_chunks=((int)ch_duration_secs)/5;
      
      printf("Number of full 5 sec chunks = %d ",number_full_chunks);
      printf(" with remainder %5.1lf sec\n",ch_duration_secs - (double)(number_full_chunks*5));
      
      blocksize=ch_samplerate*5*ch_channels*(ch_bitspersample/8);
      
      printf("Block size [bytes] from header info = %d\n",blocksize);
      
      printf("    End of header results\n\n");
    }
  }
  else
  {
    printf("  ***  File is NOT Flac format!  ***\n");
    chok=0;
  }
  fclose(infile);
  
  return chok;
}

int getbytes(int nb)
{
  int nbread;
  
  nbread=fread(chbuffer,1,nb,infile);
  chbuffer[nb+1]=(char)0;
  return nbread;
  
}

